-------------------------------------------------------------------------------
-- DKCrutch Gui functions
--
-- GUI functions and structures for DKCrutch
-------------------------------------------------------------------------------

local L = DKCrutch.Locals

function DKCrutch:CreateCheckButton(name, parent, table, field, radio)
	local button
	if radio then
		button = CreateFrame('CheckButton', parent:GetName() .. name, parent, 'SendMailRadioButtonTemplate')
	else
		button = CreateFrame('CheckButton', parent:GetName() .. name, parent, 'OptionsCheckButtonTemplate')
	end
	local frame = _G[button:GetName() .. 'Text']
	frame:SetText(name)
	frame:SetTextColor(1, 1, 1, 1)
	frame:SetFontObject(GameFontNormal)
	button:SetScript("OnShow", 
		function (self) 
			self:SetChecked(table[field]) 
			self.origValue = table[field] or self.origValue
		end 
	)
	if radio then
		button:SetScript("OnClick", 
			function (self, button, down)
				this:SetChecked(1)
				table[field] = not table[field]
			end 
		)
	else
		button:SetScript("OnClick", 
			function (self, button, down) 
				table[field] = not table[field]
			end
		)
	end

	function button:Restore() 
		table[field] = self.origValue 
	end 
	return button 
end

function DKCrutch:CreateSlider(text, parent, low, high, step)
	local name = parent:GetName() .. text
	local slider = CreateFrame('Slider', name, parent, 'OptionsSliderTemplate')
	slider:SetScript('OnMouseWheel', Slider_OnMouseWheel)
	slider:SetMinMaxValues(low, high)
	slider:SetValueStep(step)
	slider:EnableMouseWheel(true)
	_G[name .. 'Text']:SetText(text)
	_G[name .. 'Low']:SetText('')
	_G[name .. 'High']:SetText('')
	local text = slider:CreateFontString(nil, 'BACKGROUND')
	text:SetFontObject('GameFontHighlightSmall')
	text:SetPoint('LEFT', slider, 'RIGHT', 7, 0)
	slider.valText = text
	return slider
end

function DKCrutch:CreateButton(text, parent)
	local name = parent:GetName() .. text
	local button = CreateFrame('Button', name, parent, 'UIPanelButtonTemplate')
	_G[name .. 'Text']:SetText(text)
	local text = button:CreateFontString(nil, 'BACKGROUND')
	text:SetFontObject('GameFontHighlightSmall')
	text:SetPoint('LEFT', button, 'RIGHT', 7, 0)
	button.valText = text
	return button
end

function DKCrutch:CreateDropDownMenu(text, parent, dbTree, varName, itemList, width)
	local name = parent:GetName() .. text
    local menu = CreateFrame("Frame", name, parent, "UIDropDownMenuTemplate");
    menu.displayMode = "MENU"

	local frame = _G[menu:GetName() .. 'Text']
	frame:SetText(text)
	frame:SetTextColor(1, 1, 1, 1)
	frame:SetFontObject(GameFontNormal)

    menu:EnableMouse(true);
    if(width) then
        _G.UIDropDownMenu_SetWidth(menu, width);
    end
    menu.itemList = itemList or {};
    menu.init = function()
            for i=1, #menu.itemList do
                if(not menu.itemList[i].hooked) then
                    local func = menu.itemList[i].func or function(self) end;
                    menu.itemList[i].func = function(self, arg1, arg2)
                        self = self or _G.this; -- wotlk/tbc hack
                        dbTree[varName] = self.value;
                        _G.UIDropDownMenu_SetSelectedValue(menu, self.value);
                        func(self, arg1, arg2);
                    end
                    menu.itemList[i].hooked = true;
                end
                local info = _G.UIDropDownMenu_CreateInfo();
                for k,v in pairs(menu.itemList[i]) do
                    info[k] = v;
                end
                _G.UIDropDownMenu_AddButton(info, _G.UIDROPDOWNMENU_MENU_LEVEL);
            end
        end
    menu:SetScript("OnShow", function(self)
            _G.UIDropDownMenu_Initialize(self, self.init);
            _G.UIDropDownMenu_SetSelectedValue(self, dbTree[varName]);
        end);
    menu.SetValue = function(self, value)
            _G.UIDropDownMenu_SetSelectedValue(self, value);
        end;
    menu:Hide(); menu:Show();
    return menu;
end

function DKCrutch:ApplySettings()
	DKCrutch:InitSettings()
	if (not DKCrutchDB.locked) then
		DKCrutch:UnLockFrames()
	else
		if (DKCrutch.displayFrame) then
			DKCrutch.displayFrame:EnableMouse(false)
			DKCrutch.displayFrame:SetMovable(false)
			DKCrutch.displayFrame:SetBackdropColor(0, 0, 0, .0)
			DKCrutch.advisorFrame:EnableMouse(false)
			DKCrutch.advisorFrame:SetMovable(false)
			DKCrutch.advisorFrame:SetBackdropColor(0, 0, 0, .0)
		end
	end
	if ( DKCrutchDB.enabled ) then
		DKCrutch.eventFrame:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
		DKCrutch.eventFrame:RegisterEvent("RUNE_POWER_UPDATE")
		DKCrutch.eventFrame:RegisterEvent("RUNE_TYPE_UPDATE")
		if (DKCrutch.displayFrame) then
			DKCrutch.displayFrame:Show()
			if (DKCrutchDB.advisorDisabled) then
			else
				DKCrutch.advisorFrame:Show()
			end
		end
	else
		DKCrutch.eventFrame:UnregisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
		DKCrutch.eventFrame:UnregisterEvent("RUNE_POWER_UPDATE")
		DKCrutch.eventFrame:UnregisterEvent("RUNE_TYPE_UPDATE")
		if (DKCrutch.displayFrame) then
			DKCrutch.displayFrame:Hide()
			DKCrutch.advisorFrame:Hide()
		end
	end
	if (DKCrutch.displayFrame) then
		DKCrutch.displayFrame:SetAlpha(DKCrutchDB.runes.alpha)
		DKCrutch.displayFrame:SetScale(DKCrutchDB.runes.scale)
		DKCrutch.advisorFrame:SetAlpha(DKCrutchDB.advisor.alpha)
		DKCrutch.advisorFrame:SetScale(DKCrutchDB.advisor.scale)
	end
end

function DKCrutch:StoreUIValues()
    for i,v in pairs(DKCrutchDB) do
		DKCrutch.prevDB[i]=v
    end
end

function DKCrutch:ReStoreUIValues()
    for i,v in pairs(DKCrutch.prevDB) do
		DKCrutchDB[i]=v
    end
end

function DKCrutch:CreateConfig()
	DKCrutch.configPanel = CreateFrame( "Frame", "DKCrutchConfigPanel", UIParent );
	-- Register in the Interface Addon Options GUI
	-- Set the name for the Category for the Options Panel
	DKCrutch.configPanel.name = "DKCrutch";
	
	local EnableBtn = DKCrutch:CreateCheckButton(L.CONFIG_ENABLED, DKCrutch.configPanel, DKCrutchDB, "enabled", false)
	EnableBtn:SetPoint('TOPLEFT', 10, -8) 

	local LockBtn = DKCrutch:CreateCheckButton(L.CONFIG_LOCK_FRAMES, DKCrutch.configPanel, DKCrutchDB, "locked", false)
	LockBtn:SetPoint('TOPLEFT', 10, -58) 

	local Scale = DKCrutch:CreateSlider(L.CONFIG_SPELL_ADV_SCALE, DKCrutch.configPanel, .25, 3, .1)
	Scale:SetScript('OnShow', function(self)
		self.onShow = true
		self:SetValue(DKCrutchDB.advisor.scale)
		self.onShow = nil
	end)
	Scale:SetScript('OnValueChanged', function(self, value)
		self.valText:SetText(format('%.1f', value))
		if not self.onShow then
			DKCrutchDB.advisor.scale=value
			DKCrutch.displayFrame:SetScale(value)
			
		end
	end)
	Scale:SetPoint("TOPLEFT",10,-98)
	Scale:Show()

	local Alpha = DKCrutch:CreateSlider(L.CONFIG_SPELL_ADV_ALPHA, DKCrutch.configPanel, .0, 1, .1)
	Alpha:SetScript('OnShow', function(self)
		self.onShow = true
		self:SetValue(DKCrutchDB.advisor.alpha)
		self.onShow = nil
	end)
	Alpha:SetScript('OnValueChanged', function(self, value)
		self.valText:SetText(format('%.1f', value))
		if not self.onShow then
			DKCrutchDB.advisor.alpha=value
			DKCrutch.advisorFrame:SetAlpha(value)
		end
	end)
	Alpha:SetPoint("TOPLEFT",200,-98)
	Alpha:Show()

	local advisorDisableBtn = DKCrutch:CreateCheckButton(L.CONFIG_DISABLE_ADVISOR, DKCrutch.configPanel, DKCrutchDB, "advisorDisabled", false)
	advisorDisableBtn:SetPoint('TOPLEFT', 10, -158) 

	local RuneScale = DKCrutch:CreateSlider(L.CONFIG_RUNE_TRACKER_SCALE, DKCrutch.configPanel, .25, 3, .1)
	RuneScale:SetScript('OnShow', function(self)
		self.onShow = true
		self:SetValue(DKCrutchDB.runes.scale)
		self.onShow = nil
	end)
	RuneScale:SetScript('OnValueChanged', function(self, value)
		self.valText:SetText(format('%.1f', value))
		if not self.onShow then
			DKCrutchDB.runes.scale=value
			DKCrutch.displayFrame:SetScale(value)
		end
	end)
	RuneScale:SetPoint("TOPLEFT",10,-138)
	RuneScale:Show()

	local RuneAlpha = DKCrutch:CreateSlider(L.CONFIG_RUNE_TRACKER_ALPHA, DKCrutch.configPanel, .0, 1, .1)
	RuneAlpha:SetScript('OnShow', function(self)
		self.onShow = true
		self:SetValue(DKCrutchDB.runes.alpha)
		self.onShow = nil
	end)
	RuneAlpha:SetScript('OnValueChanged', function(self, value)
		self.valText:SetText(format('%.1f', value))
		if not self.onShow then
			DKCrutchDB.runes.alpha=value
			DKCrutch.displayFrame:SetAlpha(value)
		end
	end)
	RuneAlpha:SetPoint("TOPLEFT",200,-138)
	RuneAlpha:Show()

	local ResetBtn = DKCrutch:CreateButton(L.CONFIG_RESET_POSITIONS, DKCrutch.configPanel)
	ResetBtn:SetWidth(160)
	ResetBtn:SetHeight(22)
	ResetBtn:SetScript('OnClick', function()
		DKCrutch:ResetPosition()
	end)
	ResetBtn:SetPoint("TOPLEFT",10,-398)
	ResetBtn:Show()
	
	DKCrutch.configPanel.okay = function()
		DKCrutch:ApplySettings()
	end
	DKCrutch.configPanel.cancel = function()
		-- cancel button pressed, revert changes
		DKCrutch:ReStoreUIValues()
		DKCrutch:ApplySettings()
	end
	DKCrutch.configPanel.default = function()
		-- default button pressed, reset setting
		DKCrutch:ResetDB(true)
		DKCrutch:ResetPosition()
	end

	-- always show frame if config panel is open
	DKCrutch.configPanel:SetScript('OnShow', function(self)
		self.onShow = true
		self.onShow = nil
	end)
	DKCrutch.configPanel:SetScript('OnHide', function(self)
		self.onHide = true
		self.onHide = nil
	end)
	-- Add the panel to the Interface Options
	InterfaceOptions_AddCategory(DKCrutch.configPanel)
end

function DKCrutch:ResetDB()
	if (not DKCrutchDB.runes) or (force) then
		DKCrutchDB.runes = {
			["x"] = -180,
			["y"] = 0,
			["scale"] = 1,
			["alpha"] = 1,
			["relativePoint"] = "CENTER",
		}
	end
	if (not DKCrutchDB.advisor) or (force) then
		DKCrutchDB.advisor = {
			["x"] = 180,
			["y"] = 0,
			["scale"] = 1,
			["alpha"] = 1,
			["relativePoint"] = "CENTER",
		}
	end

	if (force) then
		DKCrutchDB.locked = false
		DKCrutchDB.enabled = true
		DKCrutchDB.advisorDisabled = false
	end
end

function DKCrutch.Options(msg)
	if (msg=='debug') then
		if (DKCrutch.DebugMode) then
			DKCrutch:Debug("Debug ended", GetTime())
		end
		DKCrutch.DebugMode = not ( DKCrutch.DebugMode )
		local debugStatus = "disabled"
		if (DKCrutch.DebugMode) then
			DKCrutch.DebugChat = DKCrutch:GetDebugFrame()
			debugStatus = "enabled. Using frame: " .. DKCrutch.DebugChat:GetID()
			DKCrutch:Debug("Debug started", GetTime())
		end
		DEFAULT_CHAT_FRAME:AddMessage("DKCrutch Debug " .. debugStatus)
	else
		InterfaceOptionsFrame_OpenToCategory(getglobal("DKCrutchConfigPanel"))
	end
end

function DKCrutch:adjustDeathTracker()
	if DKCrutch.deathTracker then
		if (DKCrutch.talent == "frost") then
			DKCrutch.deathTracker:SetStatusBarTexture("Interface\\AddOns\\DKCrutch\\media\\death.tga")
			DKCrutch.deathTracker:SetMinMaxValues(0,20)
		else
			DKCrutch.deathTracker:SetStatusBarTexture("Interface\\AddOns\\DKCrutch\\media\\death4.tga")
			DKCrutch.deathTracker:SetMinMaxValues(0,40)
		end
	end
end

function DKCrutch:ResetPosition()
	DKCrutchDB.runes.x = 0
	DKCrutchDB.runes.y = -100
	DKCrutchDB.runes.relativePoint = "CENTER"
	DKCrutchDB.advisor.x = -100
	DKCrutchDB.advisor.y = 0
	DKCrutchDB.advisor.relativePoint = "CENTER"

	DKCrutch.displayFrame:ClearAllPoints()
	DKCrutch.displayFrame:SetPoint(DKCrutchDB.runes.relativePoint,DKCrutchDB.runes.x,DKCrutchDB.runes.y)
	DKCrutch.advisorFrame:ClearAllPoints()
	DKCrutch.advisorFrame:SetPoint(DKCrutchDB.advisor.relativePoint,DKCrutchDB.advisor.x,DKCrutchDB.advisor.y)

end

function DKCrutch:MakeDraggable(frame,store)
	frame:Show()
	frame:SetBackdropColor(0, 0, 0, .3)
	frame:EnableMouse(true)
	frame:SetMovable(true)
	frame:SetClampedToScreen(true)
	frame:SetScript("OnMouseDown", function(self) self:StartMoving(); self:SetBackdropColor(0, 0, 0, .6); end)
	frame:SetScript("OnMouseUp", function(self)
		self:StopMovingOrSizing()
		if (DKCrutch.locked) then
			self:SetBackdropColor(0, 0, 0, 0)
		else
			self:SetBackdropColor(0, 0, 0, .3)
		end
		local _,_,rp,x,y = self:GetPoint()
		DKCrutchDB[store]["x"] = x
		DKCrutchDB[store]["y"] = y
		DKCrutchDB[store]["relativePoint"] = rp
	end)
	frame:SetScript("OnDragStop", function(self)
		self:StopMovingOrSizing();
		if (DKCrutch.locked) then
			self:SetBackdropColor(0, 0, 0, 0)
		else
			self:SetBackdropColor(0, 0, 0, .3)
		end
		local _,_,rp,x,y = self:GetPoint()
		DKCrutchDB[store]["x"] = x
		DKCrutchDB[store]["y"] = y
		DKCrutchDB[store]["relativePoint"] = rp
	end)
end

function DKCrutch:UnLockFrames()
	DKCrutch:MakeDraggable(DKCrutch.displayFrame,"runes")
	DKCrutch:MakeDraggable(DKCrutch.advisorFrame,"advisor")
end

function DKCrutch:CreateGUI()
	local displayFrame = CreateFrame("Frame","DKCrutchDisplayFrame",UIParent)
	displayFrame:SetFrameStrata("BACKGROUND")
	displayFrame:SetWidth(84)
	displayFrame:SetHeight(64)
	displayFrame:SetBackdrop({
          bgFile = "Interface\\Tooltips\\UI-Tooltip-Background", tile = true, tileSize = 32,
	})
	displayFrame:SetBackdropColor(0, 0, 0, .0)
	displayFrame:SetAlpha(DKCrutchDB.runes.alpha)
	displayFrame:SetPoint(DKCrutchDB.runes.relativePoint,DKCrutchDB.runes.x,DKCrutchDB.runes.y)
	
	local iconTexture = displayFrame:CreateTexture(nil,"BACKGROUND")
	iconTexture:SetTexture("Interface\\AddOns\\DKCrutch\\media\\icons.tga", true)
	iconTexture:SetWidth(16)
	iconTexture:SetHeight(64)
	iconTexture:SetPoint('TOPLEFT',displayFrame,0,0)

	for i=1,3,1 do
		local runes = CreateFrame("StatusBar", nil, displayFrame)
		if (i==1) then
			runes:SetStatusBarTexture("Interface\\AddOns\\DKCrutch\\media\\unholy.tga")
		elseif (i==2) then
			runes:SetStatusBarTexture("Interface\\AddOns\\DKCrutch\\media\\frost.tga")
		elseif (i==3) then
			runes:SetStatusBarTexture("Interface\\AddOns\\DKCrutch\\media\\blood.tga")
		end
		runes:SetWidth(64)
		runes:SetHeight(16)
		runes:SetMinMaxValues(0,20)
		runes:SetPoint('TOPLEFT',displayFrame,20,-i*16)
	
		DKCrutch.runeTracker[i] = runes
	end
	
	local deathRunes = CreateFrame("StatusBar", nil, displayFrame)
	deathRunes:SetStatusBarTexture("Interface\\AddOns\\DKCrutch\\media\\death.tga")
	deathRunes:SetWidth(64)
	deathRunes:SetHeight(16)
	deathRunes:SetMinMaxValues(0,20)
	deathRunes:SetPoint('TOPLEFT',displayFrame,20,0)

	DKCrutch.deathTracker = deathRunes
	
	DKCrutch:adjustDeathTracker()
	
	local advisorFrame = CreateFrame("Frame", "DKCrutchAdvisorFrame", UIParent)
	advisorFrame:SetFrameStrata("BACKGROUND")
	advisorFrame:SetWidth(64)
	advisorFrame:SetHeight(64)
	advisorFrame:SetBackdrop({
          bgFile = "Interface\\Tooltips\\UI-Tooltip-Background", tile = true, tileSize = 32,
	})
	advisorFrame:SetBackdropColor(0, 0, 0, .0)

	local t = advisorFrame:CreateTexture(nil,"BACKGROUND")
	t:SetTexture("")
	t:SetAllPoints(advisorFrame)
	advisorFrame.texture = t
	advisorFrame:SetAlpha(DKCrutchDB.advisor.alpha)
	advisorFrame:SetPoint(DKCrutchDB.advisor.relativePoint,DKCrutchDB.advisor.x,DKCrutchDB.advisor.y)

	DKCrutch.advisorFrame = advisorFrame

	DKCrutch.displayFrame = displayFrame

	if (not DKCrutchDB.locked) then
		DKCrutch:UnLockFrames()
	end
end
